const BIGINT_ZERO = BigInt(0);
const BIGINT_ONE = BigInt(1);

function euclidean(a, b) {
    let x;
    let y;
    let d;
    solve(a, b);
    return [x, y, d];
    function solve(_a, _b) {
        if (_b === 0) {
            x = 1;
            y = 0;
            d = _a;
        }
        else {
            const A = _a % _b;
            const B = _b;
            solve(B, A);
            const X = y;
            const Y = x;
            x = X;
            y = Y - Math.floor(_a / _b) * X;
        }
    }
}
function euclideanBigint(a, b) {
    let x;
    let y;
    let d;
    solve(a, b);
    return [x, y, d];
    function solve(_a, _b) {
        if (_b === BIGINT_ZERO) {
            x = BIGINT_ONE;
            y = BIGINT_ZERO;
            d = _a;
        }
        else {
            const A = _a % _b;
            const B = _b;
            solve(B, A);
            const X = y;
            const Y = x;
            x = X;
            y = Y - (_a / _b) * X;
        }
    }
}

function gcd(x, y) {
    let i = x;
    for (let j = y; j !== 0;) {
        const t = i;
        i = j;
        j = t % i;
    }
    return i;
}
function gcdBigint(x, y) {
    let i = x;
    for (let j = y; j !== BIGINT_ZERO;) {
        const t = i;
        i = j;
        j = t % i;
    }
    return i;
}

export { euclidean, euclideanBigint, gcd, gcdBigint };
